﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Text;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Martian_WinForms
{
    public partial class Form1 : Form
    {
        [DllImport("kernel32.dll")]
        static extern IntPtr GetConsoleWindow();

        [DllImport("user32.dll")]
        static extern bool ShowWindow(IntPtr hWnd, int nCmdShow);

        const int SW_HIDE = 0;
        public static int GUI_Height = 32;
        Font font;
        int screenWidth = Common.CurrentMap.GetWidth * Settings.BlockSize;
        int screenHeight = Common.CurrentMap.GetHeight * Settings.BlockSize;

        public Form1()
        {
            InitializeComponent();
            InitializeGame();

            if (!Settings.ConsoleMode)
            {
                var handle = GetConsoleWindow();
                ShowWindow(handle, SW_HIDE);
            }
            if (Common.GameOver)
            {
                Application.Exit();
            }
        }

        private void InitializeGame()
        {
            Size = new Size(screenWidth + 16, screenHeight + 39 + GUI_Height);
            pictureBox1.Size = new Size(screenWidth, screenHeight);
            font = new Font("Arial", 16, FontStyle.Bold);
            Common.Effects = new List<Effect>
            {
                new Effect("Blood", Color.Red, 0.5),
                new Effect("Hit", Color.White, 0.20)
            };
            Common.Popups = new List<Popup>();
            timer1.Start();
        }

        private void Draw(object sender, PaintEventArgs e)
        {
            for (int X = 0; X < Common.CurrentMap.mapTiles.GetLength(0); X++)
            {
                for (int Y = 0; Y < Common.CurrentMap.mapTiles.GetLength(1); Y++)
                {
                    var currentTile = Common.CurrentMap.mapTiles[X, Y];
                    e.Graphics.DrawImage(
                        Common.Tileset,
                        X * Settings.BlockSize,
                        Y * Settings.BlockSize,
                        new Rectangle(currentTile.OffsetOnTextureSheet, 0, Settings.BlockSize, Settings.BlockSize),
                        GraphicsUnit.Pixel);
                    int textOffsetX = 4;
                    int textOffsetY = 4;
                    // Рисовать текст поверх тех блоков, над которыми требуется.
                    if (currentTile.Letter != char.MinValue && currentTile.TileType != TileType.Fire)
                    { 
                        e.Graphics.DrawString(currentTile.Letter.ToString(),
                            font,
                            Brushes.White,
                            X * Settings.BlockSize + textOffsetX,
                            Y * Settings.BlockSize + textOffsetY);
                    }
                }
            }
            foreach (var instance in Movable.AllEntities)
            {
                e.Graphics.DrawImage(
                    instance.SpriteImage,
                    instance.GetActualX,
                    instance.GetActualY,
                    new Rectangle(instance.XSpriteOffset, 0, 32, 32),
                    GraphicsUnit.Pixel);
            }
            // Визуализация текущего пути.
            if (Settings.DisplayPathFinding && Settings.AIMode)
            {
                for (int X = 0; X < Common.CurrentMap.GetWidth; X++)
                {
                    for (int Y = 0; Y < Common.CurrentMap.GetHeight; Y++)
                    {
                        int count = Common.Player.CurrentPathPoints.Count(p => p.X == X && p.Y == Y);
                        if (count > 0) e.Graphics.DrawString(".", font, Brushes.White, X * Settings.BlockSize + 9, Y * Settings.BlockSize - 2);
                    }
                }
            }
            // Рисование всплывающих значков.
            foreach (var popup in Common.Popups)
            {
                e.Graphics.DrawImage(
                    Common.PopupsImages,
                    popup.X,
                    popup.Y,
                    new Rectangle(popup.offsetOnSpriteSheet, 0, 32, 32),
                    GraphicsUnit.Pixel);
                popup.Continue();
            }
            Common.Popups = Common.Popups.Where(p => p.OpacityFactor > 0).ToList();
            // Рисование эффекта поверх экрана.
            foreach (var effect in Common.Effects)
            {
                if (effect.TimeFactor > 0.05)
                    e.Graphics.FillRectangle(new SolidBrush(Color.FromArgb((int)(200 / effect.duration * effect.TimeFactor), effect.color)), new Rectangle(0, 0, screenWidth, screenHeight));
                effect.Continue();
            }
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            if (Common.Player.IsMoving || Settings.AIMode) return;
            switch (e.KeyCode)
            {
                case Keys.Up:
                    Common.Player.Move(Direction.Up);
                    break;
                case Keys.Right:
                    Common.Player.Move(Direction.Right);
                    break;
                case Keys.Down:
                    Common.Player.Move(Direction.Down);
                    break;
                case Keys.Left:
                    Common.Player.Move(Direction.Left);
                    break;
            }
        }

        private void Timer1_Tick(object sender, EventArgs e)
        {
            if (!Common.GameStarted) return;
            pictureBox1.Invalidate();
            foreach (var entity in Movable.AllEntities)
            {
                entity.Act();
                entity.MoveTowardsNext();
            }
            Common.Player.MoveTowardsNext();
            if (Settings.AIMode) Common.Player.MoveAI();
            label1.Text = "Steps: " + Common.Player.PathWalked.Length + " HP: " + Common.Player.Hp;
        }
    }
}
