﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Forms;

namespace Martian_WinForms
{
    public class Map
    {
        public List<Monster> Monsters { get; set; } = new List<Monster>();

        public List<Cube> Cubes { get; set; } = new List<Cube>();

        public Tile this[int x, int y]
        {
            get
            {
                return mapTiles[x, y];
            }
            set
            {
                mapTiles[x, y] = value;
            }
        }

        public readonly Dictionary<string, Tile> tiles = new Dictionary<string, Tile>
        {
            ["Wall"] = new Tile("Wall", TileType.Solid, 0, char.MinValue),
            ["Floor"] = new Tile("Floor", TileType.Walkable, 32, char.MinValue),
            ["Exit"] = new Tile("Exit", TileType.Walkable, 64, 'Q', TileActions.SetGameOver),
            ["Door"] = new Tile("Door", TileType.Walkable, 96, char.MinValue),
            ["Start"] = new Tile("Start", TileType.Walkable, 32, 'S'),

            ["KeyA"] = new Tile("KeyA", TileType.Key, 128, 'A', TileActions.GivePlayerKey),
            ["KeyB"] = new Tile("KeyB", TileType.Key, 128, 'B', TileActions.GivePlayerKey),
            ["KeyC"] = new Tile("KeyC", TileType.Key, 128, 'C', TileActions.GivePlayerKey),
            ["KeyD"] = new Tile("KeyD", TileType.Key, 128, 'D', TileActions.GivePlayerKey),
            ["KeyE"] = new Tile("KeyE", TileType.Key, 128, 'E', TileActions.GivePlayerKey),

            ["DoorA"] = new Tile("DoorA", TileType.Door, 96, 'A', (key) => Common.Player.HasKey(key)),
            ["DoorB"] = new Tile("DoorB", TileType.Door, 96, 'B', (key) => Common.Player.HasKey(key)),
            ["DoorC"] = new Tile("DoorC", TileType.Door, 96, 'C', (key) => Common.Player.HasKey(key)),
            ["DoorD"] = new Tile("DoorD", TileType.Door, 96, 'D', (key) => Common.Player.HasKey(key)),
            ["DoorE"] = new Tile("DoorE", TileType.Door, 96, 'E', (key) => Common.Player.HasKey(key)),

            ["Fire20"] = new Tile("Fire20", TileType.Fire, 160, 'A', (key) => TileActions.HurtPlayer(key)),
            ["Fire40"] = new Tile("Fire40", TileType.Fire, 160, 'B', (key) => TileActions.HurtPlayer(key)),
            ["Fire60"] = new Tile("Fire60", TileType.Fire, 160, 'C', (key) => TileActions.HurtPlayer(key)),
            ["Fire80"] = new Tile("Fire80", TileType.Fire, 160, 'D', (key) => TileActions.HurtPlayer(key)),
            ["Fire100"] = new Tile("Fire100", TileType.Fire, 160, 'E', (key) => TileActions.HurtPlayer(key)),

            ["HealthKit"] = new Tile("HealthKit", TileType.HealthKit, 192, char.MinValue, TileActions.HealPlayer),

            ["PlateDoorA"] = new Tile("PlateDoorA", TileType.Door, 96, 'A'),
            ["PlateDoorB"] = new Tile("PlateDoorB", TileType.Door, 96, 'B'),
            ["PlateDoorC"] = new Tile("PlateDoorC", TileType.Door, 96, 'C'),
            ["PlateDoorD"] = new Tile("PlateDoorD", TileType.Door, 96, 'D'),
            ["PlateDoorE"] = new Tile("PlateDoorE", TileType.Door, 96, 'E'),

            ["PlateA"] = new Tile("PlateA", TileType.Plate, 128, 'A'),
            ["PlateB"] = new Tile("PlateB", TileType.Plate, 128, 'B'),
            ["PlateC"] = new Tile("PlateC", TileType.Plate, 128, 'C'),
            ["PlateD"] = new Tile("PlateD", TileType.Plate, 128, 'D'),
            ["PlateE"] = new Tile("PlateE", TileType.Plate, 128, 'E'),

            ["PortalF"] = new Tile("PortalF", TileType.Portal, 224, 'F', (key) => TileActions.TeleportPlayer(key)),
            ["PortalG"] = new Tile("PortalG", TileType.Portal, 224, 'G', (key) => TileActions.TeleportPlayer(key)),
            ["PortalH"] = new Tile("PortalH", TileType.Portal, 224, 'H', (key) => TileActions.TeleportPlayer(key)),
            ["PortalI"] = new Tile("PortalI", TileType.Portal, 224, 'I', (key) => TileActions.TeleportPlayer(key)),
            ["PortalJ"] = new Tile("PortalJ", TileType.Portal, 224, 'J', (key) => TileActions.TeleportPlayer(key)),

            ["Portalf"] = new Tile("Portalf", TileType.Portal, 256, 'F', (key) => TileActions.TeleportPlayer(key, true)),
            ["Portalg"] = new Tile("Portalg", TileType.Portal, 256, 'G', (key) => TileActions.TeleportPlayer(key, true)),
            ["Portalh"] = new Tile("Portalh", TileType.Portal, 256, 'H', (key) => TileActions.TeleportPlayer(key, true)),
            ["Portali"] = new Tile("Portali", TileType.Portal, 256, 'I', (key) => TileActions.TeleportPlayer(key, true)),
            ["Portalj"] = new Tile("Portalj", TileType.Portal, 256, 'J', (key) => TileActions.TeleportPlayer(key, true)),
        };

        public readonly Tile[,] mapTiles;

        public string[,] mapTilesText;

        public bool IsPlatePressed(char key)
        {
            var plateTile = GetPointOfTile("Plate" + key);

            return Cubes.Where(c => c.X == plateTile.X && c.Y == plateTile.Y).Count() > 0;
        }

        public Point GetPointOfTile(string tileName)
        {
            for (int Y = 0; Y < GetHeight; Y++)
            {
                for (int X = 0; X < GetWidth; X++)
                {
                    if (mapTiles[X, Y].Name == tileName)
                        return new Point(X, Y);
                }
            }
            return null;
        }
        public Map(string Path)
        {
            string mapText = File.ReadAllText(Path);
            string[] mapTextLines = mapText.Split('\n');
            int Width;
            int Height;
            try
            {
                Width = Convert.ToInt32(mapTextLines[0].Split(' ')[1]);
                Height = Convert.ToInt32(mapTextLines[0].Split(' ')[0]);
                if (Width > 50 || Height > 50 || Width < 10 || Height < 10)
                {
                    Messaging.WriteEndGameMessage("Размер карты должен быть не менее 10 блоков и не более 50 в любом направлении."); 
                    return;
                }
            }
            catch (Exception e) { Messaging.WriteEndGameMessage("Ошибка при чтении файла карты: " + e.Message); return; }
            mapTiles = new Tile[Width, Height];
            mapTilesText = new string[Width, Height];
            for (int Y = 1; Y <= Height; Y++)
            {
                string[] mapLineBlocks = mapTextLines[Y].Split();
                for (int X = 0; X < Width; X++)
                {
                    mapTilesText[X, Y - 1] = mapLineBlocks[X];
                    switch (mapLineBlocks[X])
                    {
                        case "X": mapTiles[X, Y - 1] = tiles["Wall"]; break;
                        case "S": mapTiles[X, Y - 1] = tiles["Start"]; Common.Player = new Player(X, Y - 1, Common.PlayerImage); break;
                        case ".": mapTiles[X, Y - 1] = tiles["Floor"]; break;
                        case "Q": mapTiles[X, Y - 1] = tiles["Exit"]; break;
                        case "a": mapTiles[X, Y - 1] = tiles["PlateA"]; break;
                        case "b": mapTiles[X, Y - 1] = tiles["PlateB"]; break;
                        case "c": mapTiles[X, Y - 1] = tiles["PlateC"]; break;
                        case "d": mapTiles[X, Y - 1] = tiles["PlateD"]; break;
                        case "e": mapTiles[X, Y - 1] = tiles["PlateE"]; break;
                        case "A": mapTiles[X, Y - 1] = tiles["PlateDoorA"]; break;
                        case "B": mapTiles[X, Y - 1] = tiles["PlateDoorB"]; break;
                        case "C": mapTiles[X, Y - 1] = tiles["PlateDoorC"]; break;
                        case "D": mapTiles[X, Y - 1] = tiles["PlateDoorD"]; break;
                        case "E": mapTiles[X, Y - 1] = tiles["PlateDoorE"]; break;
                        case "1": mapTiles[X, Y - 1] = tiles["Fire20"]; break;
                        case "2": mapTiles[X, Y - 1] = tiles["Fire40"]; break;
                        case "3": mapTiles[X, Y - 1] = tiles["Fire60"]; break;
                        case "4": mapTiles[X, Y - 1] = tiles["Fire80"]; break;
                        case "5": mapTiles[X, Y - 1] = tiles["Fire100"]; break;
                        case "M": mapTiles[X, Y - 1] = tiles["Floor"]; Monsters.Add(new Monster(X, Y - 1, Common.MonsterImage)); break;
                        case "K": mapTiles[X, Y - 1] = tiles["Floor"]; Cubes.Add(new Cube(X, Y - 1, Common.CubeImage)); break;

                        case "F": mapTiles[X, Y - 1] = tiles["PortalF"]; break;
                        case "G": mapTiles[X, Y - 1] = tiles["PortalG"]; break;
                        case "H": mapTiles[X, Y - 1] = tiles["PortalH"]; break;
                        case "I": mapTiles[X, Y - 1] = tiles["PortalI"]; break;
                        case "J": mapTiles[X, Y - 1] = tiles["PortalJ"]; break;

                        case "f": mapTiles[X, Y - 1] = tiles["Portalf"]; break;
                        case "g": mapTiles[X, Y - 1] = tiles["Portalg"]; break;
                        case "h": mapTiles[X, Y - 1] = tiles["Portalh"]; break;
                        case "i": mapTiles[X, Y - 1] = tiles["Portali"]; break;
                        case "j": mapTiles[X, Y - 1] = tiles["Portalj"]; break;

                        default:
                            Messaging.WriteEndGameMessage("Неверный формат карты, обнаружен неизвестный блок: " + mapLineBlocks[X]);
                            mapTiles = null;
                            return;
                    }
                }
            }
        }
        public int GetWidth
        {
            get
            {
                return mapTiles.GetLength(0);
            }
        }
        public int GetHeight
        {
            get
            {
                return mapTiles.GetLength(1);
            }
        }
        public bool IsWall(int X, int Y)
        {
            return mapTiles[X, Y].TileType == TileType.Solid;
        }

        public bool HasMonsters(int X, int Y)
        {
            return Monsters.Where(m => m.X == X && m.Y == Y).Count() > 0;
        }

        public Cube GetCubeAt(int X, int Y)
        {
            return Cubes.Where(m => (m.X == X && m.Y == Y || m.NextX == X && m.NextY == Y) && !m.IsMoving).FirstOrDefault();
        }

        public bool IsLockedDoor(int X, int Y)
        {
            switch (mapTiles[X, Y].Name)
            {
                case "DoorA": return !Common.Player.HasKey('A');
                case "DoorB": return !Common.Player.HasKey('B');
                case "DoorC": return !Common.Player.HasKey('C');
                case "DoorD": return !Common.Player.HasKey('D');
                case "DoorE": return !Common.Player.HasKey('E');
                default: return false;
            }
        }

        public bool IsLockedPlateDoor(int X, int Y)
        {
            if (mapTiles[X, Y].Name.StartsWith("PlateDoor"))
            {
                var plateForDoor = GetPointOfTile("Plate" + mapTiles[X, Y].Letter);
                if (plateForDoor == null) return false;
                return Cubes.Where(c => c.X == plateForDoor.X && c.Y == plateForDoor.Y || 
                    c.NextX == plateForDoor.X && c.NextY == plateForDoor.Y).Count() == 0;
            }
            return false;
        }

        public bool IsDeadlyFire(int X, int Y)
        {
            switch (mapTiles[X, Y].Name)
            {
                case "Fire20":
                    if (Common.Player.Hp <= 20)
                        return true;
                    break;
                case "Fire40":
                    if (Common.Player.Hp <= 40)
                        return true;
                    break;
                case "Fire60":
                    if (Common.Player.Hp <= 60)
                        return true;
                    break;
                case "Fire80":
                    if (Common.Player.Hp <= 80)
                        return true;
                    break;
                case "Fire100": return true;
            }
            return false;
        }
        public Point GetNearestTilePoint(Point point, Tile tile)
        {
            var tilePoints = new List<Point>();
            for (int X = 0; X < GetWidth; X++)
                for (int Y = 0; Y < GetHeight; Y++)
                    if (mapTiles[X, Y] == tile)
                        tilePoints.Add(new Point(X, Y));
            return tilePoints.OrderBy(p => Path.GetHeuristicPathLength(new Point(point.X, point.Y), new Point(p.X, p.Y))).FirstOrDefault();
        }
        public Tile GetTileAt(int X, int Y)
        {
            return mapTiles[X, Y];
        }
        public bool IsBlock(int X, int Y, Tile tile)
        {
            return mapTiles[X, Y] == tile;
        }
        public List<Point> GetAllTilePoints(TileType tileType)
        {
            var tilePoints = new List<Point>();
            for (int X = 0; X < GetWidth; X++)
                for (int Y = 0; Y < GetHeight; Y++)
                    if (mapTiles[X, Y].TileType == tileType)
                        tilePoints.Add(new Point(X, Y));
            return tilePoints;
        }
    }
}
