﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Martian_WinForms
{
    public class Path
    {
        private readonly Point initialStart;
        private readonly Point finalGoal;
        private readonly List<Point> checkedBlockedPoints = new List<Point>();
        private readonly List<Point> checkedDeadlyFirePoints = new List<Point>();
        public int pathLength = 0;

        public Path(Point start, Point goal)
        {
            initialStart = start;
            finalGoal = goal;
        }

        public SinglyLinkedList<Point> BuildPossiblePath()
        {
            var path = FindPath(initialStart, finalGoal);
            var map = Common.CurrentMap;
            var fireCount = map.GetAllTilePoints(TileType.Fire).Count;
            while (fireCount == 0 || checkedDeadlyFirePoints.Count < fireCount)
            {
                var pathDone = true;
                if (path == null)
                {
                    Messaging.WriteEndGameMessage("Путь до выхода, к сожалению, не был найден. Вероятно, выход заблокирован стеной.");
                    return null;
                }
                var pathReverse = path.ToList();
                pathReverse.Reverse();
                // Проверка пути на препятствия.
                for(int i = 0; i < pathReverse.Count; i++)
                {
                    var point = pathReverse[i];
                    if (point.Equals(Common.Player.Position)) continue;
                    if (map.IsLockedDoor(point.X, point.Y))
                    {
                        var tile = map.mapTiles[point.X, point.Y];
                        var keyToOpenTheDoor = map.GetPointOfTile("Key" + tile.Letter);
                        if (keyToOpenTheDoor == null) 
                            Messaging.WriteEndGameMessage("На карте отсутствует ключ для одной из дверей, которую нужно пройти для решения задачи.");
                        checkedBlockedPoints.Add(point);
                        path = FindPath(initialStart, map.GetPointOfTile("Key" + tile.Letter));
                        pathDone = false;
                        break;
                    }

                    if (map.IsDeadlyFire(point.X, point.Y))
                    {
                        checkedDeadlyFirePoints.Add(point);
                        var healthKits = map.GetAllTilePoints(TileType.HealthKit);
                        foreach (var e in healthKits)
                        {
                            var pathToHealthKit = FindPath(initialStart, e);
                            if (pathToHealthKit != null) path = pathToHealthKit;
                        }
                        pathDone = false;
                        break;
                    }

                    // Если есть куб, который нужно отодвинуть.
                    var cubeAtPoint = map.GetCubeAt(point.X, point.Y);
                    if (cubeAtPoint != null &&
                        cubeAtPoint.ShouldBePushedFromSide == Direction.None &&
                        i >= 1 && 
                        i < pathReverse.Count - 1)
                    {
                        var nextPoint = pathReverse[i + 1];
                        var prevPoint = pathReverse[i - 1];
                        for (int dx = -1; dx <= 1; dx++)
                        {
                            for (int dy = -1; dy <= 1; dy++)
                            {
                                if (dx != 0 && dy != 0) continue;
                                var newPoint = new Point(cubeAtPoint.NextX + dx, cubeAtPoint.NextY + dy);
                                var oppositePoint = new Point(cubeAtPoint.NextX - dx, cubeAtPoint.NextY - dy);

                                if (nextPoint.Equals(newPoint) || prevPoint.Equals(newPoint)) continue;
                                if (FindPath(initialStart, oppositePoint) == null ||
                                    Common.CurrentMap.IsWall(newPoint.X, newPoint.Y)) continue;

                                if (dx == -1) cubeAtPoint.ShouldBePushedFromSide = Direction.Right;
                                if (dx == 1) cubeAtPoint.ShouldBePushedFromSide = Direction.Left;
                                if (dy == 1) cubeAtPoint.ShouldBePushedFromSide = Direction.Down;
                                if (dy == -1) cubeAtPoint.ShouldBePushedFromSide = Direction.Up;
                            }
                        }
                        Point position = new Point(cubeAtPoint.X, cubeAtPoint.Y);
                        switch (cubeAtPoint.ShouldBePushedFromSide)
                        {
                            case Direction.Left: position = new Point(cubeAtPoint.X - 1, cubeAtPoint.Y); break;
                            case Direction.Up: position = new Point(cubeAtPoint.X, cubeAtPoint.Y - 1); break;
                            case Direction.Right: position = new Point(cubeAtPoint.X + 1, cubeAtPoint.Y); break;
                            case Direction.Down: position = new Point(cubeAtPoint.X, cubeAtPoint.Y + 1); break;
                        }
                        path = FindPath(initialStart, position);
                        pathDone = false;
                        break;
                    }
                    // Если на пути дверь, которую нужно разблокировать.
                    if (map.IsLockedPlateDoor(point.X, point.Y))
                    {
                        var tile = map.GetTileAt(point.X, point.Y);
                        var plate = map.GetPointOfTile("Plate" + tile.Letter);
                        Cube cube = null;
                        var done = false;
                        for (int dx = -1; dx <= 1; dx++)
                        {
                            for (int dy = -1; dy <= 1; dy++)
                            {
                                if (dx != 0 && dy != 0) continue;
                                cube = map.GetCubeAt(plate.X + dx, plate.Y + dy);
                                if (cube == null) continue;
                                if (plate.X < cube.X)
                                {
                                    cube.ShouldBePushedFromSide = Direction.Right;
                                    done = true;
                                }
                                if (plate.X > cube.X)
                                {
                                    cube.ShouldBePushedFromSide = Direction.Left;
                                    done = true;
                                }

                                if (cube.Y < plate.Y)
                                {
                                    cube.ShouldBePushedFromSide = Direction.Up;
                                    done = true;
                                }
                                if (cube.Y > plate.Y)
                                {
                                    cube.ShouldBePushedFromSide = Direction.Down;
                                    done = true;
                                }
                                if (done) break;
                            }
                            if (done) break;
                        }
                        if (cube == null)
                        {
                            Messaging.WriteEndGameMessage("На карте есть неоткрываемая дверь.");
                            return null;
                        }
                        Point position = new Point(cube.X, cube.Y);
                        switch (cube.ShouldBePushedFromSide)
                        {
                            case Direction.Left: position = new Point(cube.X - 1, cube.Y); break;
                            case Direction.Up: position = new Point(cube.X, cube.Y - 1); break;
                            case Direction.Right: position = new Point(cube.X + 1, cube.Y); break;
                            case Direction.Down: position = new Point(cube.X, cube.Y + 1); break;
                        }
                        path = FindPath(initialStart, position);
                        pathDone = false;
                        break;
                    }
                }
                // Путь был успешно проверен!
                if (pathDone) return path;
            }
            Messaging.WriteEndGameMessage("На карте присутствует непроходимый огонь.");
            return null;
        }

        public SinglyLinkedList<Point> FindPath(Point start, Point goal)
        {
            var queue = new Queue<Point>();
            var visited = new HashSet<Point>();
            var result = new Dictionary<Point, SinglyLinkedList<Point>>();

            visited.Add(start);
            queue.Enqueue(start);
            result.Add(start, new SinglyLinkedList<Point>(start));

            while (queue.Count != 0)
            {
                var point = queue.Dequeue();

                if (point.X < 0 || point.X >= Common.CurrentMap.GetWidth || point.Y < 0 || point.Y >= Common.CurrentMap.GetHeight) continue;

                if (Common.CurrentMap.IsWall(point.X, point.Y)) continue;

                for (var dy = -1; dy <= 1; dy++)
                    for (var dx = -1; dx <= 1; dx++)
                    {
                        if (dx != 0 && dy != 0) continue;
                        var nextPoint = new Point(point.X + dx, point.Y + dy);

                        if (visited.Contains(nextPoint) ||
                            checkedDeadlyFirePoints.Contains(nextPoint) ||
                            checkedBlockedPoints.Contains(nextPoint))
                            continue;

                        var checkedTileForPortal = Common.CurrentMap.GetTileAt(point.X, point.Y);
                        if (checkedTileForPortal.TileType == TileType.Portal)
                        {
                            var tile = Common.CurrentMap.GetPointOfTile("Portal" + checkedTileForPortal.Letter.ToString().ToLower());
                            var newPoint = new Point(tile.X, tile.Y);
                            visited.Add(nextPoint);
                            queue.Enqueue(newPoint);
                            result.Add(newPoint, new SinglyLinkedList<Point>(nextPoint, result[point]));
                            continue;
                        }

                        // Куб, который можно толкнуть.
                        var cube = Common.CurrentMap.GetCubeAt(point.X, point.Y);
                        if (cube != null && !IsCorrectDirection(cube, nextPoint)) continue;

                        queue.Enqueue(nextPoint);
                        visited.Add(nextPoint);
                        result.Add(nextPoint, new SinglyLinkedList<Point>(nextPoint, result[point]));
                        if (nextPoint.Equals(goal)) return result[nextPoint];
                    }
            }
            return null;
        }

        private static bool IsCorrectDirection(Cube cube, Point point)
        {
            switch (cube.ShouldBePushedFromSide)
            {
                case Direction.Left: return point.X < cube.X;
                case Direction.Up: return point.Y < cube.Y;
                case Direction.Right: return point.X > cube.X;
                case Direction.Down: return point.Y > cube.Y;
            }
            return true;
        }

        public static int GetHeuristicPathLength(Point from, Point to)
        {
            return Math.Abs(from.X - to.X) + Math.Abs(from.Y - to.Y);
        }
    }
}
