﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Forms;

namespace Martian_WinForms
{
    public class Player : Movable
    {
        public Player(int x, int y, Bitmap spriteImage) : base(x, y, spriteImage) { Speed = 2; }

        #region Состояние и инвентарь

        private int hp = 100;
        public int Hp 
        {
            get
            {
                return hp;
            }
            set
            {
                hp = value;
                if (hp > 100) hp = 100;
                if (hp < 0) Messaging.WriteEndGameMessage("Ваш персонаж умер. Игра окончена!");
            } 
        }
        public List<char> KeysAcquired { get; set; } = new List<char>();
        public bool HasKey(char key)
        {
            return KeysAcquired.Contains(key);
        }
        #endregion

        #region Нахождение пути
        // Путь для ИИ.
        public Stack<Point> CurrentPathPoints { get; set; } = new Stack<Point>();
        public StringBuilder PathWalked { get; set; } = new StringBuilder();
        #endregion

        public void MoveAI()
        {
            if (IsMoving || Common.GameOver) return;
            if (CurrentPathPoints.Count == 0)
            {
                var cubeAround = AnyCubeAround();
                if (cubeAround != null)
                {
                    switch (cubeAround.ShouldBePushedFromSide)
                    {
                        case Direction.Left: CurrentPathPoints.Push(new Point(X + 1, Y)); break;
                        case Direction.Up: CurrentPathPoints.Push(new Point(X, Y + 1)); break;
                        case Direction.Right: CurrentPathPoints.Push(new Point(X - 1, Y)); break;
                        case Direction.Down: CurrentPathPoints.Push(new Point(X, Y - 1)); break;
                        default: CurrentPathPoints.Push(new Point(X + 1, Y)); break;
                    }
                }
                else FindPath();
            }
            var point = CurrentPathPoints.Pop();
            var direction = GetDirection(point);
            if (direction != Direction.None) PathWalked.Append(direction.ToString()[0]);
            Move(direction);
        }

        public Cube AnyCubeAround()
        {
            for (int dx = -1; dx <= 1; dx++)
            {
                for (int dy = -1; dy <= 1; dy++)
                {
                    var cube = Common.CurrentMap.GetCubeAt(X + dx, Y + dy);
                    if (cube != null) return cube;
                }
            }
            return null;
        }

        public override void Move(Direction direction)
        {
            base.Move(direction);
            var cubeInWay = (Common.CurrentMap.GetCubeAt(NextX, NextY));
            if (cubeInWay == null || cubeInWay.IsMoving) return;
            cubeInWay.Push(direction);
        }

        public override void HandleReachedTileCenter()
        {
            base.HandleReachedTileCenter();
            var tile = Common.CurrentMap.mapTiles[X, Y];

            var stepAction = tile.PlayerStepAction;
            var keyAction = tile.PlayerKeyStepAction;

            if (stepAction != null) stepAction.Invoke();
            if (keyAction != null) keyAction.Invoke(tile.Letter);

            if (tile.TileType == TileType.Key || tile.Name == "HealthKit")
            {
                Common.CurrentMap.mapTiles[X, Y] = Common.CurrentMap.tiles["Floor"];
                Common.CurrentMap.mapTilesText[X, Y] = ".";
            }

            if (Settings.ConsoleMode) ConsoleMode.RedrawTile(X, Y, true);
        }

        public void FindPath()
        {
            if (!Settings.AIMode) return;
            var exitPoint = Common.CurrentMap.GetPointOfTile("Exit");
            if (exitPoint == null)
            {
                Messaging.WriteEndGameMessage("На карте отсутствует точка выхода(Q).");
                return;
            }
            var path = new Path(new Point(X, Y), exitPoint).BuildPossiblePath();
            if (path == null) return;
            CurrentPathPoints.Clear();
            foreach (var e in path) CurrentPathPoints.Push(e);
        }

        public override void Act()
        {
            if (Common.CurrentMap.HasMonsters(X, Y))
            {
                var monster = Common.CurrentMap.Monsters.Where(m => m.X == X && m.Y == Y).First();
                if (monster.IsDead) return;
                monster.IsDead = true;
                Common.ActivateEffect("Hit");
            }
        }
    }
    public enum Direction
    {
        Up,
        Right,
        Down,
        Left,
        None,
        Any
    }
}
