﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Martian_WinForms
{
    public class Tile
    {
        public static Tile None
        {
            get
            {
                return new Tile("None", TileType.None, 0, char.MinValue);
            }
        }

        public readonly string Name;

        public readonly TileType TileType;

        public readonly char Letter = char.MinValue;

        public readonly Action PlayerStepAction;

        public readonly Action<char> PlayerKeyStepAction;

        public readonly Func<bool> PlayerStepFunction;

        public readonly int OffsetOnTextureSheet;

        public Tile(string name, TileType tileType, int offsetOnTextureSheet, char letter, Action playerStepAction)
            : this(name, tileType, offsetOnTextureSheet, letter)
        {
            PlayerStepAction = playerStepAction;
        }

        public Tile(string name, TileType tileType, int offsetOnTextureSheet, char letter, Action<char> playerStepAction)
            : this(name, tileType, offsetOnTextureSheet, letter)
        {
            PlayerKeyStepAction = playerStepAction;
        }

        public Tile(string name, TileType tileType, int offsetOnTextureSheet, char letter, Func<bool> playerStepAction)
            : this(name, tileType, offsetOnTextureSheet, letter)
        {
            PlayerStepFunction = playerStepAction;
        }

        public Tile(string name, TileType tileType, int offsetOnTextureSheet, char letter)
        {
            Name = name;
            TileType = tileType;
            OffsetOnTextureSheet = offsetOnTextureSheet;
            Letter = letter;
        }

        public static bool operator ==(Tile a, Tile b)
        {
            return a.Name == b.Name;
        }

        public static bool operator !=(Tile a, Tile b)
        {
            return !(a == b);
        }

        public override bool Equals(object tile)
        {
            return Name == (tile as Tile).Name;
        }

        public override int GetHashCode()
        {
            return Name.GetHashCode() * 42;
        }
    }

    static class TileActions
    {
        public static void SetGameOver()
        {
            Messaging.WriteEndGameMessage("Вы победили!");
        }

        public static void GivePlayerKey(char key)
        {
            Common.Player.KeysAcquired.Add(key);
        }

        public static bool CheckPlayerKey(char key)
        {
            return Common.Player.KeysAcquired.Contains(key);
        }
        public static void HurtPlayer(char letter)
        {
            switch (letter)
            {
                case 'A': Common.Player.Hp -= 20; break;
                case 'B': Common.Player.Hp -= 40; break;
                case 'C': Common.Player.Hp -= 60; break;
                case 'D': Common.Player.Hp -= 80; break;
                case 'E': Common.Player.Hp -= 100; break;
            }
            Common.ActivateEffect("Blood");
            Common.Player.FindPath();
        }

        public static void HealPlayer()
        {
            Common.ShowPopup(new Popup("Plus", Common.Player.GetActualX, Common.Player.GetActualY, 1, 1, 0));
            Common.Player.Hp = 100;
        }

        public static void TeleportPlayer(char letter, bool isOrange = false)
        {
            var tile = isOrange
                ? Common.CurrentMap.GetPointOfTile("Portal" + letter)
                : Common.CurrentMap.GetPointOfTile("Portal" + letter.ToString().ToLower());

            Common.Player.X = tile.X;
            Common.Player.Y = tile.Y;
            Common.Player.NextX = tile.X;
            Common.Player.NextY = tile.X;
            Common.Player.OffsetX = 0;
            Common.Player.OffsetY = 0;

            Common.Player.FindPath();
        }
    }

    public enum TileType
    {
        None,
        Walkable,
        HealthKit,
        Solid,
        Key,
        Door,
        Fire,
        Plate,
        Portal
    }
}
